@@ -56,7 +56,9 @@ from mercurial import (
     error,
     obsolete,  # delete if obsolete becomes enabled by default
     phases,
-    util,
+)
+from mercurial.utils import (
+    dateutil,
 )
 
 from ..util import (
@@ -1204,7 +1206,7 @@ class DetailsDialog(QDialog):
         self.datecb.toggled.connect(curdate.setEnabled)
         self.datecb.toggled.connect(lambda s: s and curdate.setFocus())
         curdate.clicked.connect( lambda: self.datele.setText(
-                hglib.tounicode(hglib.displaytime(util.makedate()))))
+                hglib.tounicode(hglib.displaytime(dateutil.makedate()))))
         if opts.get('date'):
             self.datele.setText(opts['date'])
             self.datecb.setChecked(True)
@@ -1395,7 +1397,7 @@ class DetailsDialog(QDialog):
         if self.datecb.isChecked():
             date = hglib.fromunicode(self.datele.text())
             try:
-                util.parsedate(date)
+                dateutil.parsedate(date)
             except error.ParseError as e:
                 if e.hint:
                     err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
