@@ -45,7 +45,9 @@ from .qtgui import (
 from mercurial import (
     error,
     phases,
-    util,
+)
+from mercurial.utils import (
+    procutil,
 )
 
 from ..util import (
@@ -2141,7 +2143,7 @@ class RepoWidget(QWidget):
 
         # 2. Expand internal workbench variables
         def filelist2str(filelist):
-            return ' '.join(util.shellquote(
+            return ' '.join(procutil.shellquote(
                             os.path.normpath(self.repo.wjoin(filename)))
                             for filename in filelist)
         if files is None:
