@@ -7,14 +7,35 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import error
+from __future__ import absolute_import
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import cmdcore, cmdui, csinfo, qtlib, resolve
+from .qtcore import (
+    pyqtSlot,
+)
+from .qtgui import (
+    QCheckBox,
+    QComboBox,
+    QFormLayout,
+    QLabel,
+    QMessageBox,
+    QSizePolicy,
+    QVBoxLayout,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from mercurial import (
+    error,
+    scmutil,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    cmdui,
+    csinfo,
+    qtlib,
+    resolve,
+)
 
 class UpdateWidget(cmdui.AbstractCmdWidget):
 
@@ -38,13 +59,14 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
         form.addRow(_('Update to:'), combo)
 
         # always include integer revision
-        try:
-            assert not isinstance(rev, (unicode, QString))
-            ctx = self.repo[rev]
-            if isinstance(ctx.rev(), int):  # could be None or patch name
-                combo.addItem(str(ctx.rev()))
-        except error.RepoLookupError:
-            pass
+        if rev:
+            assert not isinstance(rev, unicode)
+            try:
+                ctx = scmutil.revsymbol(self.repo, rev)
+                if isinstance(ctx.rev(), int):  # could be patch name
+                    combo.addItem(str(ctx.rev()))
+            except error.RepoLookupError:
+                pass
 
         combo.addItems(map(hglib.tounicode, hglib.namedbranches(repo)))
         tags = list(self.repo.tags()) + repo._bookmarks.keys()
@@ -131,11 +153,11 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             self.rev_combo.lineEdit().selectAll()
 
     def readSettings(self, qs):
-        self.merge_chk.setChecked(qs.value('merge').toBool())
+        self.merge_chk.setChecked(qtlib.readBool(qs, 'merge'))
         self.autoresolve_chk.setChecked(
             self.repo.ui.configbool('tortoisehg', 'autoresolve',
-                                    qs.value('autoresolve', True).toBool()))
-        self.verbose_chk.setChecked(qs.value('verbose').toBool())
+                                    qtlib.readBool(qs, 'autoresolve', True)))
+        self.verbose_chk.setChecked(qtlib.readBool(qs, 'verbose'))
 
         # expand options if a hidden one is checked
         self.optexpander.set_expanded(self.hiddenSettingIsChecked())
@@ -165,27 +187,27 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             self.commandChanged.emit()
             return
         try:
-            new_ctx = self.repo[new_rev]
+            new_ctx = scmutil.revsymbol(self.repo, new_rev)
             if not merge and new_ctx.rev() == self.ctxs[0].rev() \
                     and not new_ctx.bookmarks():
                 self.target_info.setText(_('(same as parent)'))
             else:
-                self.target_info.update(self.repo[new_rev])
+                self.target_info.update(new_ctx)
             # only show the path combo when there are multiple paths
             # and the target revision has subrepos
             showpathcombo = self.path_combo.count() > 1 and \
                 '.hgsubstate' in new_ctx
             self.path_combo_label.setVisible(showpathcombo)
             self.path_combo.setVisible(showpathcombo)
-        except (error.LookupError, error.RepoLookupError, error.RepoError):
+        except (error.LookupError, error.RepoError, EnvironmentError):
             self.target_info.setText(_('unknown revision!'))
         self.commandChanged.emit()
 
     def canRunCommand(self):
         new_rev = hglib.fromunicode(self.rev_combo.currentText())
         try:
-            new_ctx = self.repo[new_rev]
-        except (error.LookupError, error.RepoLookupError, error.RepoError):
+            new_ctx = scmutil.revsymbol(self.repo, new_rev)
+        except (error.LookupError, error.RepoError, EnvironmentError):
             return False
         return (self.discard_chk.isChecked()
                 or len(self.ctxs) == 2
@@ -203,7 +225,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
         activatebookmarkmode = self.repo.ui.config(
             'tortoisehg', 'activatebookmarks', 'prompt')
         if activatebookmarkmode != 'never':
-            bookmarks = self.repo[rev].bookmarks()
+            bookmarks = scmutil.revsymbol(self.repo, rev).bookmarks()
             if bookmarks and rev not in bookmarks:
                 # The revision that we are updating into has bookmarks,
                 # but the user did not refer to the revision by one of them
@@ -239,21 +261,23 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
                           '<i>You can disable this prompt by configuring '
                           'Settings/Workbench/Activate Bookmarks</i><p>') \
                         % (hglib.tounicode(rev), len(bookmarks)),
-                        self, bookmarks, self.repo._bookmarkcurrent).run()
+                        self, bookmarks, hglib.activebookmark(self.repo)).run()
                 if selectedbookmark:
                     rev = selectedbookmark
-                elif self.repo[rev] == self.repo[self.repo._bookmarkcurrent]:
+                elif (scmutil.revsymbol(self.repo, rev)
+                      == scmutil.revsymbol(self.repo,
+                                           hglib.activebookmark(self.repo))):
                     deactivatebookmark = qtlib.QuestionMsgBox(
                         _('Deactivate current bookmark?'),
                         _('Do you really want to deactivate the <i>%s</i> '
                           'bookmark?')
-                        % hglib.tounicode(self.repo._bookmarkcurrent))
+                        % hglib.tounicode(hglib.activebookmark(self.repo)))
                     if deactivatebookmark:
                         cmdline = ['bookmark']
                         if self.verbose_chk.isChecked():
                             cmdline += ['--verbose']
                         cmdline += ['-i',
-                                    hglib.tounicode(self.repo._bookmarkcurrent)]
+                                    hglib.tounicode(hglib.activebookmark(self.repo))]
                         return self._repoagent.runCommand(cmdline, self)
                     return cmdcore.nullCmdSession()
 
@@ -267,8 +291,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
             # revisions from the selected path. The only way to do so is
             # to temporarily set the default path to the selected path URL
             pullpath = hglib.fromunicode(
-                self.path_combo.itemData(
-                    self.path_combo.currentIndex()).toString())
+                self.path_combo.itemData(self.path_combo.currentIndex()))
             cmdline.append('--config')
             cmdline.append('paths.default=%s' % pullpath)
 
@@ -277,8 +300,9 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
         else:
             cur = self.repo.hgchangectx('.')
             try:
-                node = self.repo.hgchangectx(rev)
-            except (error.LookupError, error.RepoLookupError, error.RepoError):
+                node = self.repo.hgchangectx(
+                    scmutil.revsymbol(self.repo, rev).rev())
+            except (error.LookupError, error.RepoError, EnvironmentError):
                 return cmdcore.nullCmdSession()
             def isclean():
                 '''whether WD is changed'''
@@ -368,7 +392,7 @@ class UpdateWidget(cmdui.AbstractCmdWidget):
 
     @pyqtSlot(int)
     def _updatePathComboTooltip(self, idx):
-        self.path_combo.setToolTip(self.path_combo.itemData(idx).toString())
+        self.path_combo.setToolTip(self.path_combo.itemData(idx))
 
 
 class UpdateDialog(cmdui.CmdControlDialog):
